<?php

/**
 * Elettershop mailing platform manage class
 */
class Elettershop
{
    public function __construct()
    {
        add_action('admin_post_newsletter_subscribe', [$this, 'newsletterSubscribeCallback']);
        add_action('admin_post_nopriv_newsletter_subscribe', [$this, 'newsletterSubscribeCallback']);

        add_action('admin_post_newsletter_unsubscribe', [$this, 'newsletterUnsubscribeCallback']);
        add_action('admin_post_nopriv_newsletter_unsubscribe', [$this, 'newsletterUnsubscribeCallback']);
    }

    public function newsletterSubscribeCallback()
    {
        $params = [
            'noredirect'                    => '1',                          // mandatory
            'apikey'                        => ELETTERSHOP_API_KEY,          // mandatory
            'clientip'                      => ELETTERSHOP_CLIENT_IP,        // mandatory
            'membercontainer'               => ELETTERSHOP_MEMBER_CONTAINER, // mandatory
            'mandant'                       => ELETTERSHOP_MANDANT,          // mandatory
            'LanguageCode'                  => ELETTERSHOP_LANGUAGE_CODE,
            'email'                         => $this->getTextValue('email'),
            'FirstName'                     => $this->getTextValue('name'),
            'gender'                        => $this->getTextValue('gender'),
            'PostCode'                      => $this->getNumberValue('postal_code'),
            'city'                          => $this->getTextValue('city'),
            'LastName'                      => $this->getTextValue('LastName'),
            'PhoneNumber'                   => $this->getNumberValue('PhoneNumber'),
            'BreadAndPastry'                => $this->getCheckValue('pan_bolleria'),
            'FreshProducts'                 => $this->getCheckValue('frescos'),
            'RefrigeratedAndFrozen'         => $this->getCheckValue('refrigerados_congelados'),
            'pantry'                        => $this->getCheckValue('despensa'),
            'drinks'                        => $this->getCheckValue('bebidas'),
            'OrganicProducts'               => $this->getCheckValue('ecologicos'),
            'PersonalCare'                  => $this->getCheckValue('cuidado_personal'),
            'DrugstoreAndHouseholdProducts' => $this->getCheckValue('drogueria'),
            'ExpressDays'                   => $this->getCheckValue('dias_expres'),
            'Store'                         => $this->getTextValue('poblacion'),
        ];

        $aDataFields = [];

        foreach ($params as $field => $value) {
            $auxField = $field . '=' . urlencode($value);
            $aDataFields[] = $auxField;
        }

        $data = implode('&', $aDataFields);

        if (defined('ELETTERSHOP_LOG') && ELETTERSHOP_LOG === true) {
            $this->setLog($data);
        }

        $code = $this->elletershopSubscribe(ELETTERSHOP_HOST, WP_SITEURL, $data);

        http_response_code(intval($code));
        $this->getServiceResponse($code);
    }

    public function elletershopSubscribe($host, $referer, $data)
    {
        $socket = fsockopen($host, 80);

        fputs($socket, "POST /subscription/optin/ HTTP/1.1" . PHP_EOL);
        fputs($socket, "Host: $host" . PHP_EOL);
        fputs($socket, "Referer: $referer" . PHP_EOL);
        fputs($socket, "Content-type: application/x-www-form-urlencoded" . PHP_EOL);
        fputs($socket, "Content-length: " . strlen($data) . PHP_EOL);
        fputs($socket, "Connection: close " . PHP_EOL . PHP_EOL);

        fputs($socket, $data);

        $answer = null;

        while (!feof($socket)) {
            $answer .= fgets($socket, 100);
        }

        fclose($socket);

        return $this->getResponseCode($answer);
    }

    public function newsletterUnsubscribeCallback()
    {
        $params = [
            'noredirect'      => '1',                          // mandatory
            'apikey'          => ELETTERSHOP_API_KEY,          // mandatory
            'clientip'        => ELETTERSHOP_CLIENT_IP,        // mandatory
            'membercontainer' => ELETTERSHOP_MEMBER_CONTAINER, // mandatory
            'mandant'         => ELETTERSHOP_MANDANT,          // mandatory
            'email'           => $_REQUEST['email'],
        ];

        $aDataFields = [];

        foreach ($params as $field => $value) {
            $auxField = $field . '=' . urlencode($value);
            $aDataFields[] = $auxField;
        }

        $data = implode('&', $aDataFields);

        if (defined('ELETTERSHOP_LOG') && ELETTERSHOP_LOG === true) {
            $this->setLog($data);
        }

        $code = $this->elletershopUnsubscribe(ELETTERSHOP_HOST, WP_SITEURL, $data);

        http_response_code(intval($code));
        $this->getServiceResponse($code);
    }

    public function elletershopUnsubscribe($host, $referer, $data)
    {
        $socket = fsockopen($host, 80);

        fputs($socket, "POST /subscription/optout/ HTTP/1.1" . PHP_EOL);
        fputs($socket, "Host: $host" . PHP_EOL);
        fputs($socket, "Referer: $referer" . PHP_EOL);
        fputs($socket, "Content-type: application/x-www-form-urlencoded" . PHP_EOL);
        fputs($socket, "Content-length: " . strlen($data) . PHP_EOL);
        fputs($socket, "Connection: close " . PHP_EOL . PHP_EOL);

        fputs($socket, $data);

        $answer = null;

        while (!feof($socket)) {
            $answer .= fgets($socket, 100);
        }

        fclose($socket);

        return $this->getResponseCode($answer);
    }

    function getResponseCode($answer)
    {
        $code = explode(' ', $answer);
        $code = trim($code[1]);

        return $code;
    }

    function getServiceResponse($code)
    {
        switch ($code) {
            case '100':
            case '200':
                echo json_encode(['message' => 'Subscription successful']);
                break;
            case '101':
                echo json_encode(['message' => 'Subscription successfully confirmed']);
                break;
            case '300':
                echo json_encode(['message' => 'Withdrawal successful']);
                break;
            case '301':
                echo json_encode(['message' => 'Withdrawal successfully confirmed']);
                break;
            case '-1':
                echo json_encode(['message' => 'Unknown error']);
                break;
            case '-2':
                echo json_encode(['message' => 'Missing re-direct in recipient list-settings']);
                break;
            case '-100':
                echo json_encode(['message' => 'Error on Subscription']);
                break;
            case '-101':
                echo json_encode(['message' => 'Error on Subscription confirmation']);
                break;
            case '-102':
                echo json_encode(['message' => 'Error on Subscription: e-mail is already subscribed']);
                break;
            case '-300':
                echo json_encode(['message' => 'Error on withdrawal']);
                break;
            case '-301':
                echo json_encode(['message' => 'Error on withdrawal confirmation']);
                break;
            default:
                echo json_encode(['message' => 'General error']);
                break;
        }
    }

    public function getTextValue($field)
    {
        $result = '';

        if (isset($_REQUEST[$field]) && $_REQUEST[$field] !== '') {
            $result = $_REQUEST[$field];
        }

        return $result;
    }

    public function getNumberValue($field)
    {
        $result = 0;

        if (isset($_REQUEST[$field]) && $_REQUEST[$field] !== 0) {
            $result = intval((string) $_REQUEST[$field]);
        }

        return $result;
    }

    public function getCheckValue($field)
    {
        $result = '0';

        if (isset($_REQUEST[$field]) && $_REQUEST[$field] == 'on') {
            $result = '1';
        }

        return $result;
    }

    public function getDateValue($field)
    {
        $result = date('Y-m-d');

        if (isset($_REQUEST[$field]) && $_REQUEST[$field] !== '') {
            $result = $_REQUEST[$field];
        }

        return $result;
    }

    function setLog($message)
    {
        $logMessage = "User: " . $_SERVER['REMOTE_ADDR'] . ' - ' . date("F j, Y, g:i a") . PHP_EOL .
            "Data: " . $message . PHP_EOL .
            "-------------------------" . PHP_EOL;

        $fileName = plugin_dir_path(__FILE__) . '../log';
        if (!file_exists($fileName)) {
            mkdir($fileName, 0777, true);
        }
        $fileData = $fileName . '/log_' . date('Ymd') . '.log';
        file_put_contents($fileData, $logMessage . "\n", FILE_APPEND);
    }
}
